<?php

namespace App\Base\Redis;

use Predis\Client;
use App\Base\Pagination\Paginator;

class Redis
{
    private $redis;

    public function __construct(Client $redis)
    {
        $this->redis = $redis;
    }

    public function getClient(): Client
    {
        return $this->redis;
    }

    public function getIdentifierStatistics(string $identifier, $dayCount = 30, string $prefix = 'limit', $negative = false): array
    {
        $now = time();
        $keys = [];
        $days = [];

        for ($i = 0; $i < ($dayCount + 1); $i++) {
            $day = date('Ymd', $now - ($i*24*3600));
            $keys[] =
                $prefix.'|'.
                $identifier.'|'.
                $day;
            $days[] = date('Y-m-d', $now - ($i*24*3600));
        }

        $keys = array_reverse($keys);
        $days = array_reverse($days);

        $data = $this->redis->mget($keys);
        if ($negative) {
            $data = array_map(function ($val) { return -$val; }, $data);
        }

        $result = array_combine($days, array_map(function($v) { return intval($v); }, $data));
        if ($result === false) {
            throw new \RuntimeException("Invalid array combine");
        }

        return $result;
    }

    public function getIdentifiersDayStats(array $identifiers, \DateTime $date, string $prefix = 'limit')
    {
        $keys = [];
        $datestamp = $date->format('Ymd');
        foreach ($identifiers as $identifier) {
            $keys[] = $prefix.'|' . $identifier . '|' . $datestamp;
        }

        return count($keys) ? $this->redis->mget($keys) : [];
    }

    public function nullCounter(string $identifier, \DateTime $date, string $prefix = 'limit'): void
    {
        $key = $prefix.'|' . $identifier . '|' . $date->format('Ymd');
        $this->redis->del([$key]);
    }

    public function getStatusesForLogs(Paginator $logs): array
    {
        $txs = array_map(
            function ($item) { return $item['id']; },
            $logs->getIterator()->getArrayCopy()
        );
        $keys = array_map(
            function ($item) { return 'status|'.$item;},
            $txs
        );

        if (empty($keys)) {
            return [];
        }

        $result = array_combine($txs, $this->redis->mget($keys));
        if ($result === false) {
            throw new \RuntimeException("Invalid array combine");
        }

        return $result;
    }

    public function getStatusForMessageId(string $id): ?string
    {
        $mid = $this->redis->get('msgid-txid|'.$id);
        if ($mid === null) {
            $mid = $this->redis->get('msgid-txid|<'.$id.'>');
        }
        if ($mid === null) {
            return null;
        }

        return $this->redis->get('status|'.$mid);
    }

    public function getInfo(): array
    {
        return $this->redis->info();
    }

    public function getTopMessages(string $duration, int $top = 10): array
    {
        if ($duration === 'today') {
            $out = $this->redis->zrange('top|message|total|'.date('Ymd'), - $top, -1, 'WITHSCORES');
            arsort($out);
            return $out;
        } else { // month
            $time = time();
            $out = [];
            for ($i = 0; $i < 30; $i++) {
                $out = $this->addTopArray(
                    $out,
                    $this->redis->zrange('top|message|total|'.date('Ymd', $time - $i*24*3600), -100, -1, 'WITHSCORES')
                );
            }
            arsort($out);
            return array_slice($out, 0, $top);
        }
    }

    // type = user/ip
    public function getTopUser(string $type, string $duration, int $top = 10): array
    {
        $out = [];

        if ($duration === 'today') {
            $out = $this->redis->zrange('top|'.$type.'|total|'.date('Ymd'), - $top, -1, 'WITHSCORES');
            arsort($out);
        } else { // month
            $time = time();
            for ($i = 0; $i < 30; $i++) {
                $out = $this->addTopArray(
                    $out,
                    $this->redis->zrange('top|'.$type.'|total|'.date('Ymd', $time - $i*24*3600), -100, -1, 'WITHSCORES')
                );
            }
            arsort($out);
            $out = array_slice($out, 0, $top);
        }

        if ($type == 'ip') {
            return $out;
        }

        $user = [];
        $time = time();
        foreach ($out as $key => $total) {
            $data = [];
            $data['total'] = $total;
            $data['in'] = $data['out'] = 0;
            for ($i = 0; $i < ($duration === 'today' ? 1 : 30); $i++) {
                $date = date('Ymd', $time - $i * 24 * 3600);
                $data['in'] += intval($this->redis->zscore('top|' . $type . '|in|' . $date, $key));
                $data['out'] += intval($this->redis->zscore('top|' . $type . '|out|' . $date, $key));
            }
            $user[$key] = $data;
        }
        return $user;
    }

    private function addTopArray(array $in, array $add): array
    {
        foreach ($add as $key => $val) {
            if (!isset($in[$key])) {
                $in[$key] = 0;
            }

            $in[$key] += $add[$key];
        }

        return $in;
    }
}
